@@ -1,6 +1,30 @@
 #if 0
-# $Id: mpeginfo.py,v 1.25 2004/07/11 19:37:25 dischi Exp $
+# $Id: mpeginfo.py,v 1.33 2005/02/15 18:52:51 dischi Exp $
 # $Log: mpeginfo.py,v $
+# Revision 1.33  2005/02/15 18:52:51  dischi
+# some strange bugfix (what is this doing?)
+#
+# Revision 1.32  2005/01/21 16:37:02  dischi
+# try to find bad timestamps
+#
+# Revision 1.31  2005/01/08 12:06:45  dischi
+# make sure the buffer is big enough
+#
+# Revision 1.30  2005/01/02 14:57:27  dischi
+# detect ac3 in normal mpeg2
+#
+# Revision 1.29  2004/11/27 14:42:12  dischi
+# remove future warning
+#
+# Revision 1.28  2004/11/15 21:43:36  dischi
+# remove bad debugging stuff
+#
+# Revision 1.27  2004/11/12 18:10:45  dischi
+# add ac3 support in mpeg streams
+#
+# Revision 1.26  2004/10/04 18:06:54  dischi
+# test length of remaining buffer
+#
 # Revision 1.25  2004/07/11 19:37:25  dischi
 # o read more bytes on ts scan
 # o support for AC3 in private streams
@@ -87,13 +111,14 @@ EXT_START = 0xB5
 SEQ_END   = 0xB7
 GOP       = 0xB8
 
-SEQ_START_CODE = 0xB3
-PACK_PKT       = 0xBA
-SYS_PKT        = 0xBB
-PADDING_PKT    = 0xBE
-AUDIO_PKT      = 0xC0
-VIDEO_PKT      = 0xE0
-
+SEQ_START_CODE  = 0xB3
+PACK_PKT        = 0xBA
+SYS_PKT         = 0xBB
+PADDING_PKT     = 0xBE
+AUDIO_PKT       = 0xC0
+VIDEO_PKT       = 0xE0
+PRIVATE_STREAM1 = 0xBD
+PRIVATE_STREAM2 = 0xBf
 
 TS_PACKET_LENGTH = 188
 TS_SYNC          = 0x47
@@ -238,7 +263,7 @@ class MpegInfo(mediainfo.AVInfo):
             if len(buffer) < 1000:
                 break
             pos = buffer.find('\x00\x00\x01\xb5')
-            if pos == -1:
+            if pos == -1 or len(buffer) - pos < 5:
                 buffer = buffer[-10:]
                 continue
             ext = (ord(buffer[pos+4]) >> 4)
@@ -313,7 +338,7 @@ class MpegInfo(mediainfo.AVInfo):
         """
 	highbit = (ord(buffer[0])&0x20)>>5
 
-	low4Bytes= ((ord(buffer[0]) & 0x18) >> 3) << 30
+	low4Bytes= ((long(ord(buffer[0])) & 0x18) >> 3) << 30
 	low4Bytes |= (ord(buffer[0]) & 0x03) << 28
 	low4Bytes |= ord(buffer[1]) << 20
 	low4Bytes |= (ord(buffer[2]) & 0xF8) << 12
@@ -333,7 +358,7 @@ class MpegInfo(mediainfo.AVInfo):
         """
 	highbit = (ord(buffer[0]) >> 3) & 0x01
 
-	low4Bytes = ((ord(buffer[0]) >> 1) & 0x03) << 30
+	low4Bytes = ((long(ord(buffer[0])) >> 1) & 0x03) << 30
 	low4Bytes |= ord(buffer[1]) << 22;
 	low4Bytes |= (ord(buffer[2]) >> 1) << 15;
 	low4Bytes |= ord(buffer[3]) << 7;
@@ -405,6 +430,29 @@ class MpegInfo(mediainfo.AVInfo):
             self.sequence_header_offset = offset
             return 0
 
+        if id in (PRIVATE_STREAM1, PRIVATE_STREAM2):
+            # private stream. we don't know, but maybe we can guess later
+            add = ord(buffer[offset+8])
+            # if (ord(buffer[offset+6]) & 4) or 1:
+            # id = ord(buffer[offset+10+add])
+            if buffer[offset+11+add:offset+15+add].find('\x0b\x77') != -1:
+                # AC3 stream
+                for a in self.audio:
+                    if a.id == id:
+                        break
+                else:
+                    self.audio.append(mediainfo.AudioInfo())
+                    self.audio[-1].id = id
+                    self.audio[-1].codec = 'AC3'
+                    self.audio[-1].keys.append('id')
+            return 0
+
+        if id == SYS_PKT:
+            return 0
+        
+        if id == EXT_START:
+            return 0
+        
         return 0
 
 
@@ -437,7 +485,6 @@ class MpegInfo(mediainfo.AVInfo):
 
         # store first timestamp
         self.start = self.get_time(buffer[offset+4:])
-
         while len(buffer) > offset + 1000 and buffer[offset:offset+3] == '\x00\x00\x01':
             # read the mpeg header
             new_offset = self.ReadHeader(buffer, offset)
@@ -451,7 +498,8 @@ class MpegInfo(mediainfo.AVInfo):
                 offset = new_offset
 
                 # skip padding 0 before a new header
-                while not ord(buffer[offset+2]):
+                while len(buffer) > offset + 10 and \
+                          not ord(buffer[offset+2]):
                     offset += 1
 
             else:
@@ -547,7 +595,14 @@ class MpegInfo(mediainfo.AVInfo):
         ptsdts = ord(buffer[7]) >> 6
 
         if ptsdts and ptsdts == ord(buffer[9]) >> 4:
+            if ord(buffer[9]) >> 4 != ptsdts:
+                print 'WARNING: bad PTS/DTS, please contact us'
+                return packet_length, -1
+                
             # timestamp = self.ReadPTS(buffer[9:14])
+            high = ((ord(buffer[9]) & 0xF) >> 1)
+            med  = (ord(buffer[10]) << 7) + (ord(buffer[11]) >> 1)
+            low  = (ord(buffer[12]) << 7) + (ord(buffer[13]) >> 1)
             return packet_length, 9
             
         return packet_length, -1
@@ -624,7 +679,8 @@ class MpegInfo(mediainfo.AVInfo):
                 ackcount = 0
             else:
                 offset   += pos
-                ackcount += 1
+                if retpos != -1:
+                    ackcount += 1
             if ackcount > 10:
                 # looks ok to me
                 return retpos
@@ -675,15 +731,23 @@ class MpegInfo(mediainfo.AVInfo):
             elif adapt & 0x01:
                 # PES
                 timestamp = self.ReadPESHeader(c+offset, buffer[c+offset:], tsid)[1]
-                if timestamp != -1 and not hasattr(self, 'start'):
-                    self.get_time = self.ReadPTS
-                    timestamp     = c + offset + timestamp
-                    self.start = self.get_time(buffer[timestamp:timestamp+5])
-
-            elif adapt & 0x01:
-                # no PES, scan for something else here
-                pass
-
+                if timestamp != -1:
+                    if not hasattr(self, 'start'):
+                        self.get_time = self.ReadPTS
+                        timestamp = c + offset + timestamp
+                        self.start = self.get_time(buffer[timestamp:timestamp+5])
+                    elif not hasattr(self, 'audio_ok'):
+                        timestamp = c + offset + timestamp
+                        start = self.get_time(buffer[timestamp:timestamp+5])
+                        if abs(start - self.start) < 10:
+                            # looks ok
+                            self.audio_ok = True
+                        else:
+                            # timestamp broken
+                            del self.start
+                            if mediainfo.DEBUG:
+                                print 'Timestamp error, correcting'
+                            
             if hasattr(self, 'start') and self.start and \
                    self.sequence_header_offset and self.video and self.audio:
                 break
@@ -770,8 +834,10 @@ class MpegInfo(mediainfo.AVInfo):
         get the length in seconds, return -1 if this is not possible
         """
         end = self.get_endpos()
-        if end == -1 or self.start > end:
+        if end == -1:
             return -1
+        if self.start > end:
+            return int(((long(1) << 33) - 1 ) / 90000) - self.start + end
         return end - self.start
     
 
