@@ -1,6 +1,15 @@
 #if 0
-# $Id: riffinfo.py,v 1.30 2004/08/25 16:18:14 dischi Exp $
+# $Id: riffinfo.py,v 1.33 2005/03/15 17:50:45 dischi Exp $
 # $Log: riffinfo.py,v $
+# Revision 1.33  2005/03/15 17:50:45  dischi
+# check for corrupt avi
+#
+# Revision 1.32  2005/03/04 17:41:29  dischi
+# handle broken avi files
+#
+# Revision 1.31  2004/12/13 10:19:07  dischi
+# more debug, support LIST > 20000 (new max is 80000)
+#
 # Revision 1.30  2004/08/25 16:18:14  dischi
 # detect aspect ratio
 #
@@ -113,6 +122,7 @@ from mmpython import mediainfo
 
 # List of tags
 # http://kibus1.narod.ru/frames_eng.htm?sof/abcavi/infotags.htm
+# http://www.divx-digest.com/software/avitags_dll.html
 # File Format
 # http://www.taenam.co.kr/pds/documents/odmlff2.pdf
 
@@ -148,7 +158,7 @@ class RiffInfo(mediainfo.AVInfo):
             return
         self.valid = 1
         self.mime = 'application/x-wave'
-        
+        self.has_idx = False
         self.header = {}
         self.junkStart = None
         self.infoStart = None
@@ -159,7 +169,7 @@ class RiffInfo(mediainfo.AVInfo):
         elif self.type == 'WAVE':
             self.mime = 'application/x-wave'
         try:
-            while not self.parseRIFFChunk(file):
+            while self.parseRIFFChunk(file):
                 pass
         except IOError:
             if mediainfo.DEBUG:
@@ -171,7 +181,11 @@ class RiffInfo(mediainfo.AVInfo):
         for k in self.tag_map.keys():
             map(lambda x:self.setitem(x,self.gettable(k[0],k[1]),self.tag_map[k][x]),
                 self.tag_map[k].keys())
-        
+        if not self.has_idx:
+            _print('WARNING: avi has no index')
+            self.corrupt = 1
+            self.keys.append('corrupt')
+            
         
     def _extractHeaderString(self,h,offset,len):
         return h[offset:offset+len]
@@ -429,11 +443,11 @@ class RiffInfo(mediainfo.AVInfo):
     def parseRIFFChunk(self,file):
         h = file.read(8)
         if len(h) < 4:
-            return 1
+            return False
         name = h[:4]
         size = struct.unpack('<I',h[4:8])[0]        
 
-        if name == 'LIST' and size < 20000:
+        if name == 'LIST' and size < 80000:
             pos = file.tell() - 8
             t = file.read(size)
             key = t[:4]
@@ -455,13 +469,33 @@ class RiffInfo(mediainfo.AVInfo):
             self.junkStart = file.tell() - 8
             self.junkSize  = size
             file.seek(size, 1)
-        elif name in ('LIST', 'idx1'):
+        elif name == 'idx1':
+            self.has_idx = True
+            _print('idx1: %s bytes' % size)
             # no need to parse this
             t = file.seek(size,1)
-        else:
+        elif name == 'LIST':
+            _print('RIFF LIST to long to parse: %s bytes' % size)
+            # no need to parse this
+            t = file.seek(size,1)
+        elif name == 'RIFF':
+            _print("New RIFF chunk, extended avi [%i]" % size)
+            type = file.read(4)
+            if type != 'AVIX':
+                _print("Second RIFF chunk is %s, not AVIX, skipping", type)
+                file.seek(size-4, 1)
+            # that's it, no new informations should be in AVIX
+            return False
+        elif not name.strip(string.printable + string.whitespace):
+            # check if name is something usefull at all, maybe it is no
+            # avi or broken
             t = file.seek(size,1)
             _print("Skipping %s [%i]" % (name,size))
-        return 0
+        else:
+            # bad avi
+            _print("Bad or broken avi")
+            return False
+        return True
 
     def buildTag(self,key,value):
         text = value + '\0'
